' This program is provided as Freeware for personal, educational
' and non-commercial use under a GNU General Public License.
'
' Written by Mike Perks (basicx@austin.rr.com), 2005
'
' Example IR decoder for Sony remote
'
' This module contains the code to retrieve the IR command using the InputCapture port on ZX-24/ZX-40
'
' The SIRC-12 protocol defined by Sony has five elements:
'
' 1. An initialization pulse which is sent at the beginning of each serial string to announce the traffic 
' and help distinguish the data pulses from potential bus noise. This pulse is nominally 2400 us long.
'
' 2. A pulse representing a binary one. This pulse is nominally 1200 us long.
'
' 3. A pulse representing a binary zero. This pulse is nominally 600 us long.
'
' 4. Every zero or one pulse is preceded by a 600 us "gap".
'
' 5. The frame rate is one command every 45ms. Remotes can repeat the commands multiple times to ensure
' that it is received correctly.
'
' Sony remotes use a 7 bit command code with the least significant bit sent first,
' followed by a 5 bit device code. So for start bit, 12 gaps and 12 data bits 
' we need to collect 25 pulses.

Option Explicit
Option Strict On
  
' InputCapture is used to capture the pulse train into the array Pulses
Private Const pulsestreamLength as Integer = 25
Private pulses(1 to pulsestreamLength) as New UnsignedInteger

' CPU clock period in uS
Private Const clkPeriod_uS as Single = (1.0E+6 / 14.7456E+6) 

' use original pulse widths as cheaper than converting to microsecond equivalents
Private Const startBitLength as UnsignedInteger	= CUInt(2450.0 / clkPeriod_uS)	'2450 us
Private Const zeroBitLength as UnsignedInteger	= CUInt(625.0 / clkPeriod_uS)	'625 us
Private Const oneBitLength as UnsignedInteger	= CUInt(1225.0 / clkPeriod_uS)	'1225 us
Private Const gapLength as UnsignedInteger		= CUInt(575.0 / clkPeriod_uS)	'575 us
Private Const deltaLength as UnsignedInteger	= CUInt(100.0 / clkPeriod_uS)	'100 us

'task stack for retrieving IR commands
Private Const IRStackSize as Byte = 60
Private IRStack(1 to IRStackSize) as Byte 

' queue used for holding the command codes
Private Const queueSize as Integer = 15
Private IRQueue(1 to queueSize) as Byte

' used internally if there is an error while retrieving the IR command
Private Const errorCommand as Byte = 255

' time in seconds between repeated command codes
Private Const repeatTime as Single = 0.045


' *************************************************************
' Sub InitRemote
' 
' Initialize remote type
'**************************************************************
Public Sub InitRemote()
	' open queue
	Call OpenQueue(IRQueue, QueueSize)
	' start task
	CallTask "getIR", IRStack 
End Sub


' *************************************************************
' Function GetIRCommand
' 
' Returns next IR command if there is one
'**************************************************************
Public Const NoCommand as Byte = 255

Public Function GetIRCommand() as Byte
	' initialize to no command
	GetIRCommand = NoCommand
	
	If GetQueueCount(IRQueue) > 0 Then
		' get command codes in the queue
		Call GetQueue(IRQueue, GetIRCommand, 1)
	End If	
End Function


' *************************************************************
' Sub ExitRemote
' 
' Terminate use of remote
'**************************************************************
Public Sub ExitRemote()
	' terminate task
	Call ExitTask(IRStack)
End Sub


' *************************************************************
' Sub getIR
' 
' Get IR code and put on a queue until stopped
'**************************************************************
Private Sub getIR()
    Dim command as Byte

	' loop forever waiting for an IR command on the input capture port
    Do
 		command = fetchCommand()	
		If command <> errorCommand Then
			' put command on the queue if not full
            If GetQueueCount(IRQueue) < GetQueueBufferSize(IRQueue) Then    
                Call PutQueue(IRQueue, command, 1)
			Else
				Debug.Print "Lost IR command"
            End If       
        End If
  		Call Sleep(repeatTime)  ' remote repeats code after 45ms
    Loop
End Sub


' *************************************************************
' Function fetchCommand
' 
' Get IR code and put on a queue until stopped
'**************************************************************
Private Function fetchCommand() as Byte

    Dim n as Integer
    Dim valid as Boolean

    fetchCommand = errorCommand
    Call InputCapture (pulses, pulseStreamLength, 0)

    ' check for any timeouts in the pulse widths; i.e. for value of 65535
    valid = True
    For n = 1 To pulseStreamLength
        If pulses(n) = 65535 Then
            valid = False
		Else
			'Debug.print CStr(pulses(n))
        End If
    Next

    ' if no timeouts then try converting stream to a command code
    If valid Then
        ' check widths of gaps
         For n = 1 TO pulseStreamLength\2
           If Not withinRange(pulses(N*2), gapLength, deltaLength) Then
                valid = False
            End If
        Next

        ' check start bit
        If Not withinRange(pulses(1), startBitLength, deltaLength) Then
            valid = False
        End If
		         
        ' create 7 bit command code, LSB first 
        If valid Then
            fetchCommand = 0
            For n = 1 to 7
                If withinRange(pulses(2*n+1), zeroBitLength, deltaLength) Then
                    Call PutBit(fetchCommand, CByte(n-1), 0)
                ElseIf withinRange(pulses(2*n+1), oneBitLength, deltaLength) Then
                    Call PutBit(fetchCommand, CByte(n-1), 1)
                Else
                    valid = False 
                End If
            Next
        End If

		' skip producing device code
    End If

    ' make sure we got a valid command code
    If Not valid Then
        fetchCommand = errorCommand
    End If
End Function


' *************************************************************
' Function withinRange
' 
' Check that a number is within a certain range 
' plus or minus some delta
'**************************************************************
Private Function withinRange(ByVal number as UnsignedInteger, ByVal range as UnsignedInteger, _
							ByVal delta as UnsignedInteger) as Boolean
							
   withinRange = (CUInt(Abs(CInt(number - range))) < delta)
End Function